/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
** 
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>

extern WINDOW	*win_input, *win_info;
extern buddywin_t	*curbwin, nullbwin;
extern faimconf_t	faimconf;
extern int	iamaway, stayconnected, scrollbackoff, changetime;
extern naimses_t	naimses;
extern void	*nfire_c;
extern time_t	now;

void	conio_exit(char *params) {
	if (params != NULL) {
		cechof("/EXIT and /QUIT take no paramaters.\n");
		return;
	}
	stayconnected = 0;
}

void	conio_nick(const char *SN) {
	if ((SN == NULL) || (*SN == 0)) {
		cechof("You need to tell me who you want to be.\n");
		return;
	}
	if (strlen(SN) > MAXSNLEN) {
		cechof("Screen names must be %i characters or shorter.\n",
			MAXSNLEN);
		return;
	}
//	strcpy(naimses.sn, SN);
	firetalk_set_nickname(nfire_c, SN);
}

void	conio_info(const char *SN) {
	if (SN == NULL) {
		if (curbwin == &nullbwin) {
			cechof("You cannot get info on status.\n");
			return;
		}
		if (curbwin->ischat == 1) {
			cechof("You cannot get info on a chat room.\n");
			return;
		}
		SN = curbwin->winname; /* SN's scope is shorter then curbwin's. */
	}
	firetalk_get_info(nfire_c, SN);
}

void	conio_block(const char *SN) {
	if ((SN == NULL) || (*SN == 0)) {
		cechof("You need to supply a screenname to block.\n");
		return;
	}
	cechof("Now blocking %s.\n", SN);
	firetalk_im_add_deny(nfire_c, SN);
	bgoing(SN);
}

void	conio_unblock(const char *SN) {
	if ((SN == NULL) || (*SN == 0)) {
		cechof("You need to supply a screenname to unblock.\n");
		return;
	}
	cechof("No longer blocking %s.\n", SN);
	firetalk_im_remove_deny(nfire_c, SN);
	bcoming(SN, 0, 0);
}

#if 0
void	conio_crypt(const char *key) {
	if (curbwin == &nullbwin) {
		cechof("You cannot crypt your status window.\n");
		return;
	}
	if ((key == NULL) || (*key == 0))
		key = "naim default crypt key";
	if (strcasecmp(key, "off") == 0) {
		free(curbwin->cryptkey);
		curbwin->cryptkey = NULL;
		cechof("Crypt session ended.\n");
	} else {
		curbwin->cryptkey = strdup(key);
		cechof("Crypt session started with key=\"%s\".\n", key);
	}
}
#endif

// ischat	/invite buddy chat hi there
//		/invite buddy hi there
// isbuddy	/invite buddy chat hi there
//		/invite chat hi there
// status	/invite buddy chat hi there
void	conio_invite(char *SN, char *room, char *msg) {
	if ((curbwin != &nullbwin) && ((msg == NULL) || (*msg == 0))) {
		if (curbwin->ischat) {
		}
	}
	if ((SN == NULL) || (*SN == 0) || (room == NULL) || (*room == 0)) {
		cechof("You must supply both a buddy to invite, and a"
			" room to invite them to.\n");
		return;
	}
	firetalk_chat_invite(nfire_c, room, SN,
		((msg != NULL) && (*msg != 0))?msg:"Join me in this Buddy Chat.");
}

void	conio_op(const char *room, const char *nick) {
	if (curbwin->ischat && (nick == NULL))
		firetalk_chat_op(nfire_c, curbwin->winname, room);
	else
		firetalk_chat_op(nfire_c, room, nick);
}

void	conio_deop(const char *room, const char *nick) {
	if (curbwin->ischat && (nick == NULL))
		firetalk_chat_deop(nfire_c, curbwin->winname, room);
	else
		firetalk_chat_deop(nfire_c, room, nick);
}

void	conio_topic(const char *topic) {
	if (curbwin->ischat == 0)
		cechof("You can only change the topics of rooms.\n");
	else
		firetalk_chat_set_topic(nfire_c, curbwin->winname, topic);
}

void	conio_kick(const char *room, const char *nick, const char *why) {
	buddywin_t	*bwin;

	if (room == NULL) {
		cechof("USAGE: /kick [<room>] [<nick>] [<reason>]\n");
		return;
	}
	if (((bwin = bgetwin(room)) == NULL) || !(bwin->ischat)) {
		why = nick;
		nick = room;
		room = curbwin->winname;
	}

	if (why == NULL)
		why = naimses.sn;
	if (curbwin->ischat && (nick == NULL))
		firetalk_chat_kick(nfire_c, curbwin->winname, room, why);
	else
		firetalk_chat_kick(nfire_c, room, nick, why);
}

void	conio_join(const char *room) {
	if ((room == NULL) || (*room == 0)) {
		cechof("You must specify a room to join.\n");
		return;
	}
	firetalk_chat_join(nfire_c, room);
}

void	conio_closewin(const char *SN) {
	buddywin_t	*bwin, *bbefore;

	if (SN == NULL) {
		if ((bwin = curbwin) == &nullbwin) {
			cechof("You cannot close the status window.\n");
			return;
		}
	} else
		if ((bwin = bgetwin(SN)) == NULL) {
			cechof("No window is open for %s.\n", SN);
			return;
		}
	bbefore = bwin->next;
	while (bbefore->next != bwin)
		bbefore = bbefore->next;
	bbefore->next = bwin->next;
	if (bwin == curbwin)
		curbwin = bwin->next;
	if (bwin->ischat == 1)
		firetalk_chat_part(nfire_c, bwin->winname);
	else if ((bwin->offline == 0) && (secs_getvar_int("syncbuddy") == 1))
		firetalk_im_remove_buddy(nfire_c, bwin->winname);
	bremove(bwin);
	bupdate();
	touchwin(curbwin->win);
}

void	conio_openwin(const char *SN) {
	if ((SN == NULL) || (*SN == 0)) {
		cechof("You must supply a screenname to talk to.\n");
		return;
	}

	if (strlen(SN) > MAXSNLEN) {
		cechof("Screen names must be %i characters or shorter.\n",
			MAXSNLEN);
		return;
	}

	if (bgetwin(SN) != NULL)
		cechof("There is already a window open for %s.\n", SN);
	else {
		buddywin_t	*bwin = NULL;

		cechof("Opened window for %s.\n", SN);
		bnewwin(SN);
		bwin = bgetwin(SN);
		assert(bwin != NULL);
		bwin->offline = 0;
		bwin->notbuddy = 1;
		wechof(bwin, "Non-buddy window created.\n");
		bupdate();
		if (secs_getvar_int("syncbuddy") == 1)
			firetalk_im_add_buddy(nfire_c, SN);
	}
}

void	conio_addbuddy(const char *SN, const char *group, const char *notes) {
	if ((SN == NULL) || (*SN == 0)) {
		cechof("You must supply at least a screenname for addbuddy.\n");
		return;
	}
	if (strlen(SN) > MAXSNLEN) {
		cechof("Screen names must be %i characters or shorter.\n",
			MAXSNLEN);
		return;
	}
	if ((group == NULL) || (*group == 0))
		group = "Buddy";
	raddbuddy(&faimconf, SN, group, notes);
	if (naimses.connected == 1)
		firetalk_im_add_buddy(nfire_c, SN);
}

void	conio_save(void) {
	firetalk_save_config(nfire_c);
}

void	conio_warn(const char *SN) {
	if ((SN == NULL) || (*SN == 0)) {
		if (curbwin == &nullbwin) {
			cechof("Who do you want to warn, yourself?\n");
			return;
		}
		SN = curbwin->winname;
	}
	cechof("Eek, stay away, %s is EVIL!\n", SN);
	firetalk_im_evil(nfire_c, SN);
}

void	conio_ignore(const char *SN, const char *notes) {
	if ((SN == NULL) || (*SN == 0)) {
		cechof("You must supply at least a screenname for ignore.\n");
		return;
	}
	if (strlen(SN) > MAXSNLEN) {
		cechof("Screen names must be %i characters or shorter.\n",
			MAXSNLEN);
		return;
	}

	if (notes == NULL) {
		cechof("Now ignoring %s.\n", SN);
		notes = "";
	} else if (strcasecmp(notes, "block") == 0)
		cechof("Now blocking %s.\n", SN);
	else
		cechof("Now ignoring %s (%s).\n", SN, notes);

	raddidiot(&faimconf, SN, notes);
	if ((naimses.connected == 1) && (strcasecmp(notes, "block") == 0))
		firetalk_im_add_deny(nfire_c, SN);
}

void	conio_action(const char *msg) {
	struct tm	*tmptr = NULL;

	tmptr = localtime(&now);
	assert(tmptr != NULL);

	hwprintf(curbwin->win, IMWIN_TEXT_U,
		"[%02i:%02i:%02i] * <B>%s</B> %s<br>",
		tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec,
		naimses.sn, msg);
	naim_send_act(curbwin->winname, msg);
}

void	conio_sendim(const char *SN, const char *msg) {
	buddywin_t	*bwin = NULL;
	struct tm	*tmptr = NULL;

	if ((SN == NULL) || (*SN == 0)) {
		cechof("You must supply at least a screenname to IM.\n");
		return;
	}
	if (strlen(SN) > MAXSNLEN) {
		cechof("Screen names must be %i characters or shorter.\n",
			MAXSNLEN);
		return;
	}
	if (msg == NULL) {
		firetalk_get_info(nfire_c, SN);
		return;
	}
	tmptr = localtime(&now);
	assert(tmptr != NULL);
	if (curbwin == (bwin = bgetwin(SN))) {
		if (bwin->ischat == 1)
			hwprintf(curbwin->win, IMWIN_TEXT_U,
				"[%02i:%02i:%02i] &lt;<B>%s</B>&gt; %s<br>",
				tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec,
				naimses.sn, msg);
		else
			hwprintf(curbwin->win, IMWIN_TEXT_U,
				"[%02i:%02i:%02i] <B>%s</B>: %s<br>",
				tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec,
				naimses.sn, msg);
	} else
		hwprintf(curbwin->win, IMWIN_TEXT_U,
			"[%02i:%02i:%02i] -> *<B>%s</B>* %s<br>",
			tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec,
			SN, msg);
	logim(naimses.sn, SN, msg);
	naim_send_im(SN, msg, (bwin != NULL)?bwin->cryptkey:NULL);
}

void	conio_jump(const char *SN) {
	buddywin_t	*bwin = NULL;

	if (SN == NULL) {
		bwin = curbwin;
		while (((bwin = bwin->next) != curbwin) && (bwin->waiting == 0))
			;
	} else
		if ((bwin = bgetwin(SN)) == NULL) {
			cechof("No such buddy %s.\n", SN);
			return;
		}
	if (bwin != curbwin) {
		curbwin = bwin;
		bupdate();
		touchwin(curbwin->win);
	}
}

void	conio_reload(const char *file) {
	char	buf[256];

	killconf(&faimconf);
	if (file == NULL) {
		snprintf(buf, sizeof(buf), "%s/.faimrc", getenv("HOME"));
		file = buf;
	}
	cechof("Re-reading %s\n", file);
	naim_read_config(file);
	wsetp();
//	naim_budprof(nfaim_sess, nfaim_conn);
}

void	conio_away(const char *awaymsg) {
	if (awaymsg == NULL) {
		if (iamaway)
			unsetaway();
		else
			setaway();
		return;
	}
	secs_setvar("awaymsg", awaymsg);
	setaway();
}

int	conio_handlecmd(const char *buf) {
	char	line[1024];
	char	*cmd, *parms;

	assert(buf != NULL);
	strncpy(line, buf, sizeof(line)-1);
	line[sizeof(line)-1] = 0;
	cmd = atom(line);
	parms = firstwhite(line);
	if (script_doalias(cmd, parms) == 1)
		return(1);
	if (strcasecmp(cmd, "EVAL") == 0) {
		secs_script_parse(parms);
	} else if ((strcasecmp(cmd, "IM") == 0)
		|| (strcasecmp(cmd, "M") == 0)
		|| (strcasecmp(cmd, "MSG") == 0)) {
		conio_sendim(atom(parms), firstwhite(parms));
	} else if (strcasecmp(cmd, "CLEAR") == 0) {
		int	i;

		werase(curbwin->win);
		for (i = 0; i < faimconf.wstatus.pady; i++)
			wprintf(curbwin->win, 0, 0, "\n");
	} else if (strcasecmp(cmd, "ME") == 0) {
		conio_action(parms);
	} else if (strcasecmp(cmd, "RELOAD") == 0) {
		conio_reload(parms);
	} else if (strcasecmp(cmd, "AWAY") == 0) {
		conio_away(parms);
	} else if (strcasecmp(cmd, "SET") == 0) {
		set_setvar(parms);
	} else if ((strcasecmp(cmd, "OPEN") == 0)
		|| (strcasecmp(cmd, "WINDOW") == 0)) {
		conio_openwin(parms);
	} else if (strcasecmp(cmd, "JOIN") == 0) {
		conio_join(parms);
	} else if (strcasecmp(cmd, "OP") == 0) {
		conio_op(atom(parms), firstwhite(parms));
	} else if (strcasecmp(cmd, "DEOP") == 0) {
		conio_deop(atom(parms), firstwhite(parms));
	} else if (strcasecmp(cmd, "TOPIC") == 0) {
		conio_topic(parms);
	} else if (strcasecmp(cmd, "KICK") == 0) {
		char	*room = firstwhite(parms);

		conio_kick(atom(parms), atom(room), firstwhite(room));
	} else if (strcasecmp(cmd, "INVITE") == 0) {
		char	*room = firstwhite(parms);

		conio_invite(atom(parms), atom(room), firstwhite(room));
	} else if ((strcasecmp(cmd, "CLOSE") == 0)
		|| (strcasecmp(cmd, "ENDWIN") == 0)
		|| (strcasecmp(cmd, "PART") == 0)) {
		conio_closewin(parms);
	} else if ((strcasecmp(cmd, "ADDBUDDY") == 0)
		|| (strcasecmp(cmd, "ADD") == 0)) {
		char	*group = firstwhite(parms);

		conio_addbuddy(atom(parms), atom(group), firstwhite(group));
	} else if ((strcasecmp(cmd, "HELP") == 0)
		|| (strcasecmp(cmd, "ABOUT") == 0)) {
		help_printhelp(parms);
	} else if ((strcasecmp(cmd, "J") == 0)
		|| (strcasecmp(cmd, "JUMP") == 0)) {
		conio_jump(parms);
	} else if ((strcasecmp(cmd, "QUIT") == 0)
		|| (strcasecmp(cmd, "EXIT") == 0)) {
		conio_exit(parms);
	} else if ((strcasecmp(cmd, "INFO") == 0)
		|| (strcasecmp(cmd, "WHOIS") == 0)
		|| (strcasecmp(cmd, "WI") == 0)) {
		conio_info(parms);
	} else if (strcasecmp(cmd, "IGNORE") == 0) {
		conio_ignore(atom(parms), firstwhite(parms));
	} else if (strcasecmp(cmd, "BLOCK") == 0) {
		conio_block(parms);
	} else if (strcasecmp(cmd, "UNBLOCK") == 0) {
		conio_unblock(parms);
	} else if (strcasecmp(cmd, "SAVE") == 0) {
		conio_save();
	} else if (strcasecmp(cmd, "WARN") == 0) {
		conio_warn(parms);
	} else if (strcasecmp(cmd, "NICK") == 0) {
		conio_nick(parms);
//	} else if (strcasecmp(cmd, "CRYPT") == 0) {
//		conio_crypt(parms);
	} else
		return(0);
	return(1);
}
int      (*secs_client_cmdhandler)(const char *) = conio_handlecmd;

void	conio_handleline(char *line) {
	if (curbwin == &nullbwin) {
		if (*line == '/') {
			if (conio_handlecmd(line+1) == 0)
				conio_openwin(line+1);
		} else
			conio_sendim(atom(line), firstwhite(line));
	} else {
		if (*line == '/') {
			if (conio_handlecmd(line+1) == 0)
				cechof("Unknown command\n");
		} else {
			curbwin->keepafterso = 1;
			conio_sendim(curbwin->winname, line);
		}
	}
}

#define ADDTOBUF(ch)	{ \
	if (bufloc < sizeof(buf)-1) { \
		if (buf[bufloc] != 0) \
			memmove(buf+bufloc+1, buf+bufloc, sizeof(buf)-bufloc-1); \
		buf[bufloc++] = (ch); \
	} else \
		beep(); \
}

void	gotkey(int c) {
#ifndef NO_HISTORY
	HISTORY_STATE	*hstate = NULL;
	static int	histpos;
#endif
	static int	bufloc;
	static char	buf[1024], gotesc, inbold, initalic;

	if (c == 0) {
		bzero(buf, sizeof(buf));
		bufloc = initalic = inbold = gotesc = 0;
#ifndef NO_HISTORY
		histpos = 0;
#endif
		return;
	}
	assert(curbwin != NULL);
	if (c == 27) {
		if (!gotesc) {
			gotesc = 1;
			cechof("Press Escape again to exit.\n");
		} else {
			cechof("Logging off...\n");
//			aim_logoff(nfaim_sess);
			wshutitdown();
			exit(0);
		}
		return;
	}
	gotesc = 0;
#define CTRL(x)	((x)-'A'+1)
	switch(c) {
		case ERR:
			break;
		case KEY_UP:
#ifndef NO_HISTORY
			hstate = history_get_history_state();
			assert(hstate != NULL);
			if (hstate->entries == NULL)
				break;
			if (histpos > 0) {
				histpos--;
				assert(hstate->entries[histpos] != NULL);
				bzero(buf, sizeof(buf));
				strncpy(buf,
					(hstate->entries[histpos])->line,
					sizeof(buf)-1);
				bufloc = strlen(buf);
			}
#endif
			break;
		case KEY_DOWN:
#ifndef NO_HISTORY
			hstate = history_get_history_state();
			assert(hstate != NULL);
			if (hstate->entries == NULL)
				break;
			if (histpos < hstate->length) {
				histpos++;
				bzero(buf, sizeof(buf));
				if (hstate->entries[histpos] != NULL)
					strncpy(buf,
						(hstate->entries[histpos])->line,
						sizeof(buf)-1);
				bufloc = strlen(buf);
			}
#endif
			break;
		case KEY_LEFT:
			if (bufloc > 0)
				bufloc--;
			break;
		case KEY_RIGHT:
			if (buf[bufloc] != 0)
				bufloc++;
			break;
		case '\t': case KEY_NPAGE:
			curbwin = curbwin->next;
			bupdate();
			touchwin(curbwin->win);
			break;
	        case KEY_PPAGE: {
			buddywin_t	*bbefore = curbwin->next;

			while (bbefore->next != curbwin)
				bbefore = bbefore->next;
		        curbwin = bbefore;
                        bupdate();
                        touchwin(curbwin->win);
		        break;
		}
		case '\b': case KEY_BACKSPACE: case KEY_DC: case 0x7F:
			if (bufloc < 1)
				break;
			memmove(buf+bufloc-1, buf+bufloc,
				sizeof(buf)-bufloc);
			bufloc--;
			break;
		case '\r': case '\n':
			if (bufloc == 0)
				break;
#ifndef NO_HISTORY
			add_history(buf);
#endif
			conio_handleline(buf);
#ifndef NO_HISTORY
			hstate = history_get_history_state();
			assert(hstate != NULL);
			histpos = hstate->length;
#endif
		case CTRL('T'):
			bzero(buf, sizeof(buf));
			inbold = initalic = bufloc = 0;
			break;
		case '<':
			ADDTOBUF('&');
			ADDTOBUF('l');
			ADDTOBUF('t');
			ADDTOBUF(';');
			break;
		case '>':
			ADDTOBUF('&');
			ADDTOBUF('g');
			ADDTOBUF('t');
			ADDTOBUF(';');
			break;
		case '&':
			ADDTOBUF('&');
			ADDTOBUF('a');
			ADDTOBUF('m');
			ADDTOBUF('p');
			ADDTOBUF(';');
			break;
		case KEY_F(1): case CTRL('K'):
			help_printhelp(NULL);
			break;
		case KEY_F(2): case CTRL('B'):
			ADDTOBUF('<');
			if (inbold)
				ADDTOBUF('/');
			ADDTOBUF('B');
			ADDTOBUF('>');
			inbold = (inbold == 0);
			break;
		case KEY_F(3): case CTRL('V'):
			ADDTOBUF('<');
			if (initalic)
				ADDTOBUF('/');
			ADDTOBUF('I');
			ADDTOBUF('>');
			initalic = (initalic == 0);
			break;
		case KEY_F(4): case CTRL('Q'):
			changetime = (changetime == 0);
			break;
		case KEY_F(5):
			ADDTOBUF('&');
			break;
		case KEY_F(6): case CTRL('L'): {
				int	i;

				werase(curbwin->win);
				for (i = 0; i < faimconf.wstatus.pady; i++)
					wprintf(curbwin->win, 0, 0, "\n");
			}
			break;
		case KEY_F(7): case CTRL('R'): 
			conio_reload(NULL);
			break;
		case KEY_F(8): case CTRL('N'):
			conio_jump(NULL);
			break;
		case KEY_F(9): case CTRL('A'):
			if (iamaway)
				unsetaway();
			else
				setaway();
			break;
		case KEY_F(10): case CTRL('W'): 
			conio_closewin(NULL);
			break;
		case KEY_F(11):
			scrollbackoff += faimconf.wstatus.widthy-2;
			break;
		case KEY_F(12):
			scrollbackoff -= faimconf.wstatus.widthy-2;
			if (scrollbackoff < 0)
				scrollbackoff = 0;
			break;
		default:
			if (isprint(c)) {
				ADDTOBUF(c);
				break;
			}
	}
	werase(win_input);
	{
		int	off = (bufloc < faimconf.winput.widthx)?0:
				faimconf.winput.widthx+(faimconf.winput.widthx-10)*((bufloc-faimconf.winput.widthx)/(faimconf.winput.widthx-10))-10;

		if (buf[bufloc] == 0) {
			wprintf(win_input, INPUT_TEXT, 1, "%s", buf+off);
			wprintf(win_input, INPUT_TEXT_ALT, 1, " ");
		} else {
			int	len = bufloc-off;

			wprintf(win_input, INPUT_TEXT, 1, "%.*s", len, buf+off);
			wprintf(win_input, INPUT_TEXT_ALT, 1, "%c",
				buf[bufloc]);
			wprintf(win_input, INPUT_TEXT, 1, "%s", 
				buf+off+len+1);
		}
	}
	touchwin(win_input);
	touchwin(win_info);
}
