/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
** 
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>

extern WINDOW	*win_input, *win_buddy, *win_info;
extern buddywin_t	*curbwin, nullbwin;
extern faimconf_t	faimconf;
extern int	wsetup_called;
int	stayconnected, showrcedit = 0, gotdumped, scrollbackoff = 0;
naimses_t      naimses;
void	*nfire_c;
time_t	now, changetime;

#define statrefresh()	pnoutrefresh(curbwin->win, \
				faimconf.wstatus.pady-faimconf.wstatus.widthy-1-scrollbackoff, \
				0, \
				faimconf.wstatus.starty, \
				faimconf.wstatus.startx, \
				faimconf.wstatus.starty+faimconf.wstatus.widthy-1, \
				faimconf.wstatus.startx+faimconf.wstatus.widthx-1);

int	main(int argc, char **args) {
	time_t	lastcycle = 0;
	char	faimrcpath[256];

	now = time(NULL);
//	signal(SIGWINCH, SIG_IGN);
	if (getenv("FAIMRC") != NULL) {
		strncpy(faimrcpath, getenv("FAIMRC"),
			sizeof(faimrcpath)-1);
		faimrcpath[sizeof(faimrcpath)-1] = 0;
	} else
		snprintf(faimrcpath, sizeof(faimrcpath), "%s/.faimrc",
			getenv("HOME"));

	secs_init();
	initscr();

	wsetup_called = 2;

	sechof("Reading %s\n", faimrcpath);
	if (naim_read_config(faimrcpath) == 0) {
		wshutitdown();
		sechof("Error %i reading %s: %s\n", errno,
			faimrcpath, strerror(errno));
		exit(-1);
	} else
		endwin();
	wsetup_called = 0;
	wsetup();
	bsetup();
	gotkey(0);	// initialize gotkey() buffer
	updateidletime();
	wcols(&faimconf);
	if (faimconf.nolog == 0) {
		if (*(faimconf.logfile) == 0)
			strncpy(faimconf.logfile, "naim.log",
				sizeof(faimconf.logfile)-1);
		sechof("Opening log file (%s)\n", faimconf.logfile);
		logim(NULL, NULL, faimconf.logfile);
	} else if (*(faimconf.logfile) != 0)
		sechof("You have logging disabled, but you specified a log file.\n");
	sechof("Contact Daniel Reed <n@ml.org> if you'd like information about naim.\n");
	sechof("The latest version of naim can be downloaded from http://dev.n.ml.org/, but be sure to sign up for the naim-announce mailing list to receive release announcements (send a blank email to <naim-announce-subscribe@n.ml.org>).\n");
	sechof("If you're interested in the development of naim (including UI changes and new features), or have some ideas of your own, please subscribe to the naim-users mailing list (send a blank email to <naim-users-subscribe@n.ml.org>).\n");
	statrefresh();
	doupdate();

	naimses.online = naimses.connected = 0;
	if (*faimconf.server == 0)
		nfire_c = firetalk_create_handle(FP_AIMTOC);
	else
		nfire_c = firetalk_create_handle(FP_IRC);

	firetalk_register_callback(nfire_c, FC_DISCONNECT,
		naim_error_disconnect);
	firetalk_register_callback(nfire_c, FC_ERROR,
		naim_error_msg);
	firetalk_register_callback(nfire_c, FC_IM_GETACTION,
		naim_act_handle);
	firetalk_register_callback(nfire_c, FC_IM_GETMESSAGE,
		naim_im_handle);
	firetalk_register_callback(nfire_c, FC_EVILED,
		naim_warned);
	firetalk_register_callback(nfire_c, FC_IM_BUDDYONLINE,
		naim_buddy_coming);
	firetalk_register_callback(nfire_c, FC_IM_BUDDYOFFLINE,
		naim_buddy_going);
	firetalk_register_callback(nfire_c, FC_IDLEINFO,
		naim_buddy_idle);
	firetalk_register_callback(nfire_c, FC_IM_BUDDYAWAY,
		naim_buddy_away);
	firetalk_register_callback(nfire_c, FC_IM_BUDDYUNAWAY,
		naim_buddy_unaway);
	firetalk_register_callback(nfire_c, FC_GOTINFO,
		naim_userinfo_handler);
	firetalk_register_callback(nfire_c, FC_IM_LISTBUDDY,
		naim_buddylist);
	firetalk_register_callback(nfire_c, FC_DOINIT,
		naim_budprof);
	firetalk_register_callback(nfire_c, FC_CHAT_JOINED,
		naim_chat_joined);
	firetalk_register_callback(nfire_c, FC_CHAT_LEFT,
		naim_chat_left);
	firetalk_register_callback(nfire_c, FC_CHAT_KICKED,
		naim_chat_kicked);
	firetalk_register_callback(nfire_c, FC_CHAT_GETACTION,
		naim_chat_act_handle);
	firetalk_register_callback(nfire_c, FC_CHAT_GETMESSAGE,
		naim_chat_getmessage);
	firetalk_register_callback(nfire_c, FC_CHAT_INVITED,
		naim_chat_invited);
	firetalk_register_callback(nfire_c, FC_CHAT_USER_JOINED,
		naim_chat_JOIN);
	firetalk_register_callback(nfire_c, FC_CHAT_USER_LEFT,
		naim_chat_PART);
	firetalk_register_callback(nfire_c, FC_CHAT_USER_KICKED,
		naim_chat_KICK);
	firetalk_register_callback(nfire_c, FC_CHAT_USER_OPPED,
		naim_chat_OP);
	firetalk_register_callback(nfire_c, FC_CHAT_USER_DEOPPED,
		naim_chat_DEOP);
	firetalk_register_callback(nfire_c, FC_CHAT_GOTTOPIC,
		naim_chat_TOPIC);
	firetalk_register_callback(nfire_c, FC_USER_NICKCHANGED,
		naim_nickchange);
	firetalk_register_callback(nfire_c, FC_NEWNICK,
		naim_newnick);

	firetalk_subcode_register_request_callback(nfire_c, "VERSION",
		naim_ctcp_VERSION);
	firetalk_subcode_register_request_callback(nfire_c, "PING",
		naim_ctcp_PING);

	do {
		int	keepgoing;
		fd_set	rfd;

		sechof("Authenticating %s.\n", faimconf.screenname);
		statrefresh();
		doupdate();
		if (firetalk_signon(nfire_c, 
			(*faimconf.server)?faimconf.server:NULL, 0,
			faimconf.screenname, faimconf.password)) {
			firetalk_perror("firetalk_signon");
			statrefresh();
			doupdate();
			wshutitdown();
			exit(1);
		}

//		firetalk_list_buddies(nfire_c);

		sechof("You are now connected.\n");
		statrefresh();
		doupdate();

		naimses.lag = naimses.sn[MAXSNLEN] = 0;
		strncpy(naimses.sn, faimconf.screenname, MAXSNLEN);
		secs_setvar("SN", naimses.sn);

		debug_printf("Entering event loop.\n");
		keepgoing = stayconnected = 1;
		gotdumped = 0;

		changetime = naimses.online = time(NULL);
		naimses.connected = 1;

		while (stayconnected && keepgoing && !gotdumped) {
			struct timeval	timeout;
			time_t	now60;
			int	buddies, waiting = 0, autohide;

			now = time(NULL);
			autohide = secs_getvar_int("autohide");
			if ((now - changetime) > autohide) {
				timeout.tv_sec = 50;
				timeout.tv_usec = 0;
			} else {
				timeout.tv_sec = 0;
				timeout.tv_usec = 100000;
			}

			FD_ZERO(&rfd);
			FD_SET(STDIN_FILENO, &rfd);
			if (firetalk_select_custom(1,
				&rfd, NULL, NULL, &timeout) < 0) {
				sechof("Red alert!\n");
				wshutitdown();
				exit(0);
			}

			now = time(NULL);
			now60 = now-(now%60);
			if ((now60 - lastcycle) >= 60)
				event_handle(lastcycle = now60);

			if (FD_ISSET(STDIN_FILENO, &rfd))
				gotkey(getch());

			statrefresh();
			buddies = naimses.buddies;
			if (buddies < 0) {
				waiting = 1;
				buddies = -buddies;
			}
			if (buddies > faimconf.wbuddy.widthy)
				buddies = faimconf.wbuddy.widthy;
			if (waiting || (changetime == 0) || (autohide == 0)
				|| ((now - changetime) < autohide))
				pnoutrefresh(win_buddy,
					0, 0,
					faimconf.wbuddy.starty,
					faimconf.wbuddy.startx,
					faimconf.wbuddy.starty+buddies-1,
					faimconf.wbuddy.startx+faimconf.wbuddy.widthx-1);
			wnoutrefresh(win_info);
			wnoutrefresh(win_input);
			doupdate();
		}
		naimses.connected = 0;
		if (stayconnected)
			sechof("Connection lost, attempting to reconnect.\n");
		bclearall();
//		pnoutrefresh(win_buddy);
		curbwin = &nullbwin;
		statrefresh();
		wnoutrefresh(win_input);
		wnoutrefresh(win_info);
		doupdate();
		sechof("Closing all lingering connection.\n");
//		aim_logoff(nfaim_sess);
	} while (stayconnected);

//	aim_logoff(nfaim_sess);
	wshutitdown();
	exit(0);
}
