/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
**
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>
#include <stdio.h>
#include <fcntl.h>

#ifdef HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif

faimconf_t	faimconf;

void	killconf(faimconf_t *conf) {
	buddylist_t	*buddy = NULL;
	ignorelist_t	*idiot = NULL;

	assert(conf != NULL);
	buddy = conf->buddies;
	idiot = conf->idiots;

	while (buddy != NULL) {
		buddylist_t	*bnext = buddy->next;

		free(buddy);
		buddy = bnext;
	}
	while (idiot != NULL) {
		ignorelist_t	*inext = idiot->next;

		free(idiot);
		idiot = inext;
	}
	if (faimconf.profile != NULL)
		free(faimconf.profile);
	bzero(conf, sizeof(faimconf_t));
}

buddylist_t
	*raddbuddy(faimconf_t *conf, const char *screenname, const char *group,
	const char *notes) {
	buddylist_t	*buddy = NULL;

	buddy = conf->buddies;
	while (buddy) {
		if (!aimcmp(screenname,buddy->screenname))
			return buddy;
		buddy = buddy->next;
	}

	assert(conf != NULL);
	assert(screenname != NULL);
	assert(group != NULL);
	buddy = calloc(1, sizeof(buddylist_t));
	assert(buddy != NULL);
	strncpy(buddy->screenname, screenname,
		sizeof(buddy->screenname)-1);
	if (notes != NULL)
		strncpy(buddy->notes, notes, sizeof(buddy->notes)-1);
	if (group != NULL)
		strncpy(buddy->group, group, sizeof(buddy->group)-1);
	buddy->next = conf->buddies;
	conf->buddies = buddy;

	return(buddy);
}

void	raddidiot(faimconf_t *conf, const char *screenname, const char *notes) {
	ignorelist_t	*idiot = NULL;

	assert(conf != NULL);
	assert(screenname != NULL);
	idiot = calloc(1, sizeof(ignorelist_t));
	assert(idiot != NULL);
	strncpy(idiot->screenname, screenname,
		sizeof(idiot->screenname)-1);
	if (notes != NULL)
		strncpy(idiot->notes, notes, sizeof(idiot->notes)-1);
	else
		strncpy(idiot->notes, "There is no reason, you're just"
			" ignored!", sizeof(idiot->notes)-1);
	idiot->next = conf->idiots;
	conf->idiots = idiot;
}

buddylist_t	*rgetlist(faimconf_t *conf, const char *screenname) {
	buddylist_t	*blist = NULL;

	assert(conf != NULL);
	assert(screenname != NULL);
	blist = conf->buddies;
	assert(blist != NULL);
	do {
		if (aimcmp(blist->screenname, screenname) == 0)
			return(blist);
	} while ((blist = blist->next) != NULL);
	return(NULL);
}

char	*rmakebuddyar(faimconf_t *conf) {
	buddylist_t	*buddy = NULL;
	size_t	len = 0;
	char	*buf = NULL;

	assert(conf != NULL);
	if ((buddy = conf->buddies) == NULL)
		return(NULL);
	do {
		len += strlen(buddy->screenname)+1;
	} while ((buddy = buddy->next) != NULL);
	buf = calloc(1, len+1);
	assert(buf != NULL);

	buddy = conf->buddies;
	do {
		strcat(buf, buddy->screenname);
		strcat(buf, "&");
	} while ((buddy = buddy->next) != NULL);

	return(buf);
}

static const char	*bname(const char *s) {
	register const char	*p;

	assert(s != NULL);
	p = s + strlen(s);
	while ((p > s) && (*p != '/'))
		p--;
	return((*p=='/')?(p+1):p);
}

int	naim_eval(const char *str) {
	assert(str != NULL);
	if (*str == 0)
		return(0);
	if (strncasecmp(str, "LINES", 5) == 0) {
		str += 5;
		while ((*str != 0) && isspace(*str))
			str++;
		if (*str == 0)
			return(LINES);
		if (*str == '-') {
			str++;
			while ((*str != 0) && isspace(*str))
				str++;
			return(LINES - atoi(str));
		}
		while ((*str != 0) && isspace(*str))
			str++;
		return(atoi(str));
	}
	if (strncasecmp(str, "COLS", 4) == 0) {
		str += 4;
		while ((*str != 0) && isspace(*str))
			str++;
		if (*str == 0)
			return(COLS);
		if (*str == '-') {
			str++;
			while ((*str != 0) && isspace(*str))
				str++;
			return(COLS - atoi(str));
		}
		while ((*str != 0) && isspace(*str))
			str++;
		return(atoi(str));
	}
	return(atoi(str));
}

void	rc_initdefs(faimconf_t *conf) {
	bzero(conf, sizeof(faimconf_t));
	strncpy(conf->logfile, "naim.log", sizeof(conf->logfile)-1);
	secs_setvar("awaymsg", "I am currently away...");
	secs_setvar("statusbar_away", " (away)");
	secs_setvar("statusbar_idle", " [Idle $idle]");
	secs_setvar("statusbar_lag", " [Lag $lag]");
	secs_setvar("statusbar_query", " [Query: $cur]");
	secs_setvar("statusbar", " %I:%M%p $SN$ifaway$ifquery + * [Online $online]$iflag$ifidle * naim " NAIM_VERSION " ");
	secs_makevar("beeponim", "0", 'B');
	secs_makevar("beeponsignon", "1", 'B');
	secs_makevar("autoaway", "20", 'I');
	secs_makevar("tprint", "20", 'I');
	secs_makevar("lag", "0", 'I');
	secs_makevar("autohide", "3", 'I');
	secs_makevar("syncbuddy", "1", 'B');
	secs_makevar("autobuddy", "0", 'B');
	conf->sortby = conf->nolagcheck = conf->nobackcol = conf->nolog = 0;
	conf->rcreadonly = 1;
	conf->colors[HELP_HELP].fore = 7;
	conf->colors[HELP_HELP].back = 0;
	conf->colors[HELP_TEXT].fore = 7;
	conf->colors[HELP_TEXT].back = 0;
	conf->colors[IMWIN_BGCOLOR].fore = 0;
	conf->colors[IMWIN_BGCOLOR].back = 0;
	conf->colors[IMWIN_TEXT_U].fore = 1;
	conf->colors[IMWIN_TEXT_U].back = 0;
	conf->colors[IMWIN_TEXT_R].fore = 6;
	conf->colors[IMWIN_TEXT_R].back = 0;
	conf->colors[IMWIN_STATUS].fore = 7;
	conf->colors[IMWIN_STATUS].back = 0;
	conf->colors[INPUT_BGCOLOR].fore = 0;
	conf->colors[INPUT_BGCOLOR].back = 0;
	conf->colors[INPUT_TEXT].fore = 6;
	conf->colors[INPUT_TEXT].back = 0;
	conf->colors[INPUT_TEXT_ALT].fore = 0;
	conf->colors[INPUT_TEXT_ALT].back = 6;
	conf->colors[40].fore = 6;
	conf->colors[40].back = 4;
	conf->colors[41].fore = 6;
	conf->colors[41].back = 6;
	conf->colors[42].fore = 3;
	conf->colors[42].back = 6;
	conf->colors[43].fore = 1;
	conf->colors[43].back = 6;
	conf->colors[44].fore = 7;
	conf->colors[44].back = 6;
	conf->colors[45].fore = 6;
	conf->colors[45].back = 4;
	conf->colors[46].fore = 2;
	conf->colors[46].back = 6;
	conf->colors[47].fore = 4;
	conf->colors[47].back = 6;
	conf->colors[48].fore = 5;
	conf->colors[48].back = 6;
	conf->colors[STAT_BGCOLOR].fore = 0;
	conf->colors[STAT_BGCOLOR].back = 0;
	conf->colors[STAT_TEXT].fore = 3;
	conf->colors[STAT_TEXT].back = 0;
	conf->colors[BACKDROP].fore = 4;
	conf->colors[BACKDROP].back = 4;
	conf->colors[INFO_TEXT].fore = 7;
	conf->colors[INFO_TEXT].back = 4;

	conf->wbuddy.startx = COLS-18;
	conf->wbuddy.starty = 0;
	conf->wbuddy.widthx = 18;
	conf->wbuddy.widthy = LINES-2;
	conf->wbuddy.pady = LINES-2;
	conf->wstatus.startx = 0;
	conf->wstatus.starty = 0;
//	conf->wstatus.widthx = COLS-20;
	conf->wstatus.widthx = COLS;
	conf->wstatus.widthy = LINES-2;
	conf->wstatus.pady = 500;
	conf->winput.startx = 0;
	conf->winput.starty = LINES-1;
	conf->winput.widthx = COLS;
	conf->winput.widthy = 1;
	conf->winput.pady = 0;
	conf->winfo.startx = 0;
	conf->winfo.starty = LINES-2;
	conf->winfo.widthx = COLS;
	conf->winfo.widthy = 1;
	conf->winfo.pady = 0;
}

int	naim_read_config(const char *faimrcfile) {
	FILE	*faimrc = NULL;
	char	buf[256],	// gtkFAIM's max is 255, why differ?
		*prefix = NULL, *keyword = NULL, *arg = NULL;
	int	linenum = 0;
	struct stat	statbuf;

	rc_initdefs(&faimconf);
	if ((faimrc = fopen(faimrcfile, "r")) == NULL)
		return(0);
	while (!feof(faimrc)) {
		fgets(buf, 255, faimrc);
		if (feof(faimrc))
			break;
		linenum++;
		if ((*buf == 0) || (*buf == '#'))
			continue;
		if (((arg = firstwhite(buf)) == NULL) || (*arg == 0))
			continue;
		prefix = atom(buf);
		while ((arg[strlen(arg)-1] == '\n')
			|| (arg[strlen(arg)-1] == '\r'))
			arg[strlen(arg)-1] = 0;
		if (((keyword = firstatom(prefix, ".")) == NULL)
			|| (*keyword == 0))
			keyword = prefix;
		else
			if (strcasecmp(prefix, "naim") != 0)
				continue;

		if (strcasecmp(keyword, "screenname") == 0) {
			char	*garbage = firstwhite(arg);

			if (garbage != NULL) {
				sechof("%s[%i]: screenname must have one"
					" argument, yet you have extra"
					" (%s, %s)\n", bname(faimrcfile),
					linenum, arg, garbage);
				continue;
			}
			strncpy(faimconf.screenname, arg,
				sizeof(faimconf.screenname)-1);
			sechof("Set screen name to %s.\n",
				faimconf.screenname);
			continue;
		}
		if (strcasecmp(keyword, "password") == 0) {
			char	*garbage = firstwhite(arg);

			if (garbage != NULL) {
				sechof("%s[%i]: password must have one"
					" argument, yet you have extra"
					" (%s, %s)\n", bname(faimrcfile),
					linenum, arg, garbage);
				continue;
			}
			strncpy(faimconf.password, arg,
				sizeof(faimconf.password)-1);
			continue;
		}
		if (strcasecmp(keyword, "server") == 0) {
			strncpy(faimconf.server, arg,
				sizeof(faimconf.server)-1);
			continue;
		}
		if (strcasecmp(keyword, "awaymsg") == 0) {
			secs_setvar("awaymsg", arg);
			sechof("Set away message to \"%s\".\n",
				secs_getvar("awaymsg"));
			continue;
		}
		if (strcasecmp(keyword, "logfile") == 0) {
			firstwhite(arg);
			arg = atom(arg);
			strncpy(faimconf.logfile, arg,
				sizeof(faimconf.logfile)-1);
			sechof("Logging IM's to \"%s\".\n",
				faimconf.logfile);
			continue;
		}
		if (strcasecmp(keyword, "buddy") == 0) {
			char	*group = NULL, *notes = NULL;

			if (((group = firstwhite(arg)) == NULL)
				|| (*group == 0))
				raddbuddy(&faimconf, atom(arg), "Buddy",
					NULL);
			else if (((notes = firstwhite(group)) == NULL)
				|| (*notes == 0))
				raddbuddy(&faimconf, atom(arg),
					atom(group), NULL);
			else
				raddbuddy(&faimconf, atom(arg),
					atom(group), notes);
			continue;
		}
		if (strcasecmp(keyword, "block") == 0) {
			raddidiot(&faimconf, arg, "block");
			continue;
		}
		if (strcasecmp(keyword, "ignore") == 0) {
			char	*notes = NULL;

			if (((notes = firstwhite(arg)) == NULL)
				|| (*notes == 0))
				raddidiot(&faimconf, arg, NULL);
			else
				raddidiot(&faimconf, arg, notes);
			continue;
		}
		if (strcasecmp(keyword, "beeponim") == 0) {
			secs_setvar("beeponim", arg);
			continue;
		}
		if (strcasecmp(keyword, "beeponsignon") == 0) {
			secs_setvar("beeponsignon", arg);
			continue;
		}
		if (strcasecmp(keyword, "autoaway") == 0) {
			secs_setvar("autoaway", arg);
			continue;
		}
		if (strcasecmp(keyword, "nolog") == 0) {
			faimconf.nolog = atoi(arg);
			continue;
		}
		if (strcasecmp(keyword, "nobackcol") == 0) {
			faimconf.nobackcol = atoi(arg);
			continue;
		}
		if (strcasecmp(keyword, "nolagcheck") == 0) {
			faimconf.nolagcheck = atoi(arg);
			continue;
		}
		if (strcasecmp(keyword, "tprint") == 0) {
			secs_setvar("tprint", arg);
			continue;
		}
		if (strcasecmp(keyword, "sortby") == 0) {
			if (strcasecmp(arg, "name") == 0)
				faimconf.sortby = 1;
			else if (strcasecmp(arg, "time") == 0)
				faimconf.sortby = 2;
			continue;
		}
		if (strcasecmp(keyword, "syncbuddy") == 0) {
			secs_setvar("syncbuddy", arg);
			continue;
		}
		if (strcasecmp(keyword, "autobuddy") == 0) {
			secs_setvar("autobuddy", arg);
			continue;
		}
		if (strcasecmp(keyword, "autohide") == 0) {
			secs_setvar("autohide", arg);
			continue;
		}
		if (strcasecmp(keyword, "profile") == 0) {
			char	*tempprof = NULL;
			size_t	len = 0;
			int	proffd = 0;

			if (faimconf.profile != NULL) {
				sechof("%s[%i]: profile line"
					" specified, but I've already"
					" read in a profile!\n",
					bname(faimrcfile), linenum);
				continue;
			}
			firstwhite(arg);
			arg = atom(arg);
			if (stat(arg, &statbuf) == -1) {
				sechof("%s[%i]: unable to stat profile"
					" file: %s\n",
					bname(faimrcfile), linenum,
					strerror(errno));
				continue;
			}
			if ((len = statbuf.st_size) < 1) {
				sechof("%s[%i]: profile file too small\n",
					bname(faimrcfile), linenum);
				continue;
			}
			if ((tempprof = calloc(1, len)) == NULL) {
				sechof("%s[%i]: unable to allocate %i"
					" bytes for profile file: %s\n",
					bname(faimrcfile), linenum,
					len, strerror(errno));
				continue;
			}
			if ((proffd = open(arg, O_RDONLY)) < 0) {
				sechof("%s[%i]: unable to open profile"
					" file: %s\n",
					bname(faimrcfile), linenum,
					strerror(errno));
				free(tempprof);
				continue;
			}
			if (read(proffd, tempprof, len) < len) {
				sechof("%s[%i]: error reading in profile:"
					" %s\n", bname(faimrcfile),
					linenum, strerror(errno));
				free(tempprof);
				continue;
			}
			faimconf.profile = tempprof;
			close(proffd);
			sechof("Read profile from \"%s\".\n", arg);
			continue;
		}
		if (strcasecmp(keyword, "color") == 0) {
			char	*fore = NULL, *back = NULL;

			if (((fore = firstwhite(arg)) == NULL)
				|| (*fore == 0)) {
				sechof("%s[%i]: color keyword given "
					"without foreground color.\n",
					bname(faimrcfile), linenum);
				continue;
			}
			if (((back = firstwhite(fore)) == NULL)
				|| (*back == 0)) {
				sechof("%s[%i]: color keyword given "
					"without background color.\n",
					bname(faimrcfile), linenum);
				continue;
			}
			faimconf.colors[atoi(arg)].fore = atoi(fore);
			faimconf.colors[atoi(arg)].back = atoi(back);
			continue;
		}
		if (strcasecmp(keyword, "window") == 0) {
			char	*startx = NULL, *starty = NULL,
				*widthx = NULL, *widthy = NULL;

			if (((startx = firstwhite(arg)) == NULL)
				|| (*startx == 0)) {
				sechof("%s[%i]: window keyword given "
					"without starting x position.\n",
					bname(faimrcfile), linenum);
				continue;
			}
			if (((starty = firstwhite(startx)) == NULL)
				|| (*starty == 0)) {
				sechof("%s[%i]: window keyword given "
					"without starting y position.\n",
					bname(faimrcfile), linenum);
				continue;
			}
			if (((widthx = firstwhite(starty)) == NULL)
				|| (*widthx == 0)) {
				sechof("%s[%i]: window keyword given "
					"without x width.\n",
					bname(faimrcfile), linenum);
				continue;
			}
			if (((widthy = firstwhite(widthx)) == NULL)
				|| (*widthy == 0)) {
				sechof("%s[%i]: window keyword given "
					"without y width.\n",
					bname(faimrcfile), linenum);
				continue;
			}

			if (strcasecmp(arg, "STATUS") == 0) {
				faimconf.wstatus.startx = naim_eval(startx);
				faimconf.wstatus.starty = naim_eval(starty);
				faimconf.wstatus.widthx = naim_eval(widthx);
				faimconf.wstatus.widthy = naim_eval(widthy);
			} else if (strcasecmp(arg, "INPUT") == 0) {
				faimconf.winput.startx = naim_eval(startx);
				faimconf.winput.starty = naim_eval(starty);
				faimconf.winput.widthx = naim_eval(widthx);
				faimconf.winput.widthy = naim_eval(widthy);
			} else if (strcasecmp(arg, "BUDDY") == 0) {
				faimconf.wbuddy.startx = naim_eval(startx);
				faimconf.wbuddy.starty = naim_eval(starty);
				faimconf.wbuddy.widthx = naim_eval(widthx);
				faimconf.wbuddy.widthy = naim_eval(widthy);
			} else {
				sechof("%s[%i]: window keyword given "
					"with an invalid window name (%s).\n", 
					bname(faimrcfile), linenum, arg);
				continue;
			}
			continue;
		}

		sechof("%s[%i]: ignored keyword %s (%s)\n",
			bname(faimrcfile), linenum, keyword, arg);
	}
	return(1);
}
